#include <dlfcn.h>
#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <fcntl.h>

#include <string>

static struct LibcGet
{   
    void *f;
    LibcGet() : f(dlopen("libc.so.6",RTLD_NOW)) {}
    ~LibcGet() { dlclose(f); }
    void *operator() (const char *s) { return dlsym(f, s); }
} libc;

static volatile bool Disable = false;

extern "C" {
ssize_t read(int fd,void*buf,size_t count);
int open64(const char* f, int mode, ...);
int open(const char* f, int mode, ...);
int creat(const char*f, mode_t mode);
int system(const char* cmd);
char* fgets(char*s, int size, FILE* fp);
FILE* fopen(const char*path, const char*mode);
}

bool ok_file(const std::string& path)
{
    if(path.substr(0, 9) == "/tmp/MPQS"
    && path.find("..") == path.npos) return true;
    
    return false;
}

ssize_t read(int fd,void*buf,size_t count)
{
    /* Find the real open function */
    typedef int (*old_t)(int fd,void*buf,size_t count);
    static old_t old = NULL;
    if(!old) old = (old_t)libc("read");
    
    /* After stdin has been read, new opens are not allowed anymore */
    
    if(fd == 0)
    {
        Disable = true;
        //fprintf(stderr, "stdin read started, functions disabled\n");
        //fflush(stderr);
    }
    fprintf(stderr, "rea %d\n", fd);
    
    return old(fd,buf,count);
}

char* fgets(char*s, int size, FILE* fp)
{
    /* Find the real open function */
    typedef char* (*old_t)(char*,int,FILE*);
    static old_t old = NULL;
    if(!old) old = (old_t)libc("fgets");
    
    if(fp == stdin)
    {
        Disable = true;
        //fprintf(stderr, "stdin read started, functions disabled\n");
        //fflush(stderr);
    }
    return old(s,size,fp);
}

FILE* fopen(const char*path, const char*mode)
{
    /* Find the real fopen function */
    typedef FILE* (*old_t)(const char*,const char*);
    static old_t old = NULL;
    if(!old) old = (old_t)libc("fopen");
    
    if(!ok_file(path) && Disable) { errno = EACCES; return NULL; }

    return old(path,mode);
}


int open64(const char* f, int mode, ...)
{
    /* Find the real open64 function */
    typedef int (*old_t)(const char*,int,...);
    static old_t old = NULL;
    if(!old) old = (old_t)libc("open64");
    
    if(!ok_file(f) && Disable) { errno = EACCES; return -1; }

    if(mode & O_CREAT)
    {
        va_list args;
        va_start(args, mode);
        int p = va_arg(args, int);
        va_end(args);
        return old(f,mode,p);
    }
    return old(f,mode);
}

int creat(const char*f, mode_t mode)
{
    /* Find the real open64 function */
    typedef int (*old_t)(const char*,mode_t);
    static old_t old = NULL;
    if(!old) old = (old_t)libc("creat");
    
    if(!ok_file(f) && Disable) { errno = EACCES; return -1; }
    
    return old(f,mode);
}

int open(const char* f, int mode, ...)
{
    /* Find the real open function */
    typedef int (*old_t)(const char*,int,...);
    static old_t old = NULL;
    if(!old) old = (old_t)libc("open");
    
    if(!ok_file(f) && Disable) { errno = EACCES; return -1; }

    if(mode & O_CREAT)
    {
        va_list args;
        va_start(args, mode);
        int p = va_arg(args, int);
        va_end(args);
        return old(f,mode,p);
    }
    return old(f,mode);
}

int system(const char* cmd)
{
    fprintf(stderr, "** Access denied: to system function\n");
    errno = EACCES;
    return -1;
}

